import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InterruptedIOException;
import java.io.OutputStream;
import java.lang.management.ManagementFactory;
import java.net.ServerSocket;
import java.net.Socket;
import java.security.KeyStore;
import java.security.Security;

import javax.net.ssl.KeyManagerFactory;
import javax.net.ssl.SSLContext;
import javax.net.ssl.SSLParameters;
import javax.net.ssl.SSLServerSocket;
import javax.net.ssl.SSLServerSocketFactory;
import javax.net.ssl.SSLSession;
import javax.net.ssl.SSLSocket;
import javax.net.ssl.TrustManagerFactory;

import com.ibm.lang.management.OperatingSystemMXBean;


public class ServerJsse {
    static int N = 50, L = 100, R = 2;

    ServerJsse() {
        super();
    }

    static SSLContext sslContext;

    // Open up the commonly shared KeyStore to obtain the Trusted Certificates.
    // KeyStore is of type "JKS".  Filename is "testkeys" and password is "passphrase".
    private static void initContext() {
        try {
            if (Security.getProvider("com.ibm.jsse2.IBMJSSEProvider2") == null) {
                // IBMJSSE2 Provider is not installed, install it
                Security.addProvider(new com.ibm.jsse2.IBMJSSEProvider2());
            }

            KeyStore ks = KeyStore.getInstance("JKS");
            ks.load(new FileInputStream("testkeys"), "passphrase".toCharArray());
            KeyManagerFactory kmf = KeyManagerFactory.getInstance("IbmX509","IBMJSSE2");
            kmf.init(ks, "passphrase".toCharArray());
            TrustManagerFactory tmf = TrustManagerFactory.getInstance("PKIX","IBMJSSE2");
            tmf.init(ks);
            sslContext = SSLContext.getInstance("TLSv1.2","IBMJSSE2");
            sslContext.init(kmf.getKeyManagers(), tmf.getTrustManagers(), null);
            
           
            
        } catch (Throwable e) {
            System.out.println("Failed to read testkeys file.");
            e.printStackTrace();
            System.exit(1);
        }
    }
    
    
	public static double CPUUssage() throws IOException{
		OperatingSystemMXBean operatingSystemMXBean = (OperatingSystemMXBean) ManagementFactory.getOperatingSystemMXBean();
		double cpu = operatingSystemMXBean.getProcessCpuLoad();
		double time = operatingSystemMXBean.getProcessCpuTime();
		return cpu;
		
	}
    
    

    /* Starts the ServerJsse application.
     * @param args no parameters required
     */
    public static void main(String args[]) throws IOException {
    	CPUUssage();
        int i, j, len;

        if (args.length > 0)
            N = Integer.parseInt(args[0]);
        if (args.length > 1)
            L = Integer.parseInt(args[1]);

        byte buffer[] = new byte[L];


        System.out.println("SERVER: started.");
        
        		
        
        initContext();
        
       
        try {

            // Get and print the list of supported enabled cipher suites
            System.out.println("ServerJsse: SSL server context created.");
            String supported[] = sslContext.getSocketFactory().getSupportedCipherSuites();
            String supported2[] = {"SSL_RSA_WITH_AES_128_GCM_SHA256","SSL_RSA_WITH_AES_128_CBC_SHA256"};
           
            // Create an SSL session over port 8050
            SSLServerSocketFactory factory = sslContext.getServerSocketFactory();
            SSLServerSocket ssl_server_sock = (SSLServerSocket)factory.createServerSocket(8050);
            
            
            
            SSLParameters params = sslContext.getDefaultSSLParameters();
            params.setCipherSuites(new String[] {"SSL_RSA_WITH_AES_128_GCM_SHA256","SSL_RSA_WITH_AES_128_CBC_SHA256"});
            params.setUseCipherSuitesOrder(true);
            ssl_server_sock.setSSLParameters(params);
            //ssl_server_sock.setEnabledCipherSuites(supported2);
            

			//ssl_server_sock.setNeedClientAuth(true);
			
			
            SSLSocket ssl_sock; 
            InputStream istr;
            OutputStream ostr;
            long t;
            while (buffer[0] != -1) {
                for (int n=0; n<R; n++) {
                    try {
                        ssl_sock = (SSLSocket)(ssl_server_sock.accept());
                        ssl_sock.startHandshake();
                        System.out.println("********************* CPU usage while performing handshake: " + CPUUssage()*100 + "%");
                        SSLSession session = ssl_sock.getSession();
                        System.out.println("\nServerJsse: SSL connection established");
                        
                        System.out.println("   cipher suite:       " + session.getCipherSuite());
                    } catch (IOException se) {
                        System.out.println("\nServerJsse: client connection refused\n" + se);
                        break;
                    }

                    if (L > 0) {
                        istr = ssl_sock.getInputStream();
                        ostr = ssl_sock.getOutputStream();

                        t = System.currentTimeMillis(); 
                        for (j=0;j<N;j++) {
                            for (len = 0;;) {
                                try {
                                    if ((i = istr.read(buffer, len, L-len)) == -1) {
                                        System.out.println("ServerJsse: connection dropped by partner.");
                                        ssl_sock.close();
                                        return;
                                    }
                                } catch (InterruptedIOException e) {
                                    System.out.println("waiting");
                                    continue;
                                }
                                if ((len+=i) == L) break;
                            }
                            ostr.write(buffer, 0, L);
                        }
                        System.out.println("Messages = " + N*2 + "; Time = " +
                                           (System.currentTimeMillis() - t));
                    }
                    ssl_sock.close();
                }
            }
            ssl_server_sock.close();
            System.out.println("\nServerJsse: SSL connection closed.");
            Thread.sleep(1000);

            // plain socket benchmark
            if (L > 0) {
                // Socket bench
                ServerSocket server_sock = new ServerSocket(8050);
                Socket sock = server_sock.accept();
                System.out.println("\nServerJsse: Socket connection accepted.");
                istr = sock.getInputStream();
                ostr = sock.getOutputStream();

                t = System.currentTimeMillis();
                for (j=0;j<N;j++) {
                    for (len = 0;;) {
                        if ((i = istr.read(buffer, len, L-len)) == -1) {
                            System.out.println("ServerJsse: connection dropped by partner.");
                            return;
                        }
                        if ((len+=i) == L) break;
                    }
                    ostr.write(buffer, 0, L);
                }
                System.out.println("Messages = " + N*2 + "; Time = " +
                                   (System.currentTimeMillis() - t));
                sock.close();
                server_sock.close();
                System.out.println("\nServerJsse: Socket connection closed.");
                Thread.sleep(1000);
            }
        } catch (IOException e) {
            e.printStackTrace();
        } catch (InterruptedException f) {
            f.printStackTrace();
        }
        System.out.println("ServerJsse: terminated.");
    }
}
